==> LightAutoML-0.2.4-py3-none-any.whl: lightautoml/automl/presets/base.py <==
"""AutoML presets base class."""

import os
import shutil
from typing import Optional, Any, Sequence, Iterable

import torch
import yaml
from log_calls import record_history

from ..base import AutoML
from ...dataset.base import LAMLDataset
from ...tasks import Task
from ...utils.logging import get_logger
from ...utils.timer import PipelineTimer

logger = get_logger(__name__)

base_dir = os.path.dirname(__file__)


@record_history(enabled=False)
def upd_params(old: dict, new: dict) -> dict:
    for k in new:
        if type(new[k]) is dict and k in old and type(old[k]) is dict:
            upd_params(old[k], new[k])
        else:
            old[k] = new[k]

    return old


@record_history(enabled=False)
class AutoMLPreset(AutoML):
    """ Basic class for automl preset.

    It's almost like AutoML, but with delayed initialization.
    Initialization starts on fit, some params are inferred from data
    Preset should be defined via .create_automl method. Params should be set via yaml config
    Most usefull case - end-to-end model development, ex:

    Example:

        >>> automl = SomePreset(Task('binary'), timeout=3600)
        >>> automl.fit_predict(data, roles={'target': 'TARGET'})

    """
    _default_config_path = 'example_config.yml'

    def __init__(self, task: Task, timeout: int = 3600, memory_limit: int = 16, cpu_limit: int = 4,
                 gpu_ids: Optional[str] = 'all', verbose: int = 2,
                 timing_params: Optional[dict] = None,
                 config_path: Optional[str] = None, **kwargs: Any):
        """

        Commonly _params kwargs (ex. timing_params) set via config file (config_path argument).
        If you need to change just few params, it's possible to pass it as dict of dicts, like json
        To get available params please look on default config template. Also you can find there param description
        To generate config template call SomePreset.get_config(config_path.yml)

        Args:
            task: Task to solve.
            timeout: timeout in seconds.
            memory_limit: memory limit that are passed to each automl.
            cpu_limit: cpu limit that that are passed to each automl.
            gpu_ids: gpu_ids that are passed to each automl.
            verbose: verbosity level that are passed to each automl.
            timing_params: timing param dict. Optional.
            config_path: path to config file.
            **kwargs:

        """
        self._set_config(config_path)

        for name, param in zip(['timing_params'], [timing_params]):
            if param is None:
                param = {}
            self.__dict__[name] = {**self.__dict__[name], **param}

        self.timer = PipelineTimer(timeout, **self.timing_params)
        self.memory_limit = memory_limit
        if cpu_limit == -1:
            cpu_limit = os.cpu_count()
        self.cpu_limit = cpu_limit
        self.gpu_ids = gpu_ids
        if gpu_ids == 'all':
            self.gpu_ids = ','.join(map(str, range(torch.cuda.device_count())))
        self.task = task

        self.verbose = verbose

    def _set_config(self, path):

        if path is None:
            path = os.path.join(base_dir, self._default_config_path)

        with open(path) as f:
            params = yaml.safe_load(f)

        for k in params:
            self.__dict__[k] = params[k]

    @classmethod
    def get_config(cls, path: Optional[str] = None) -> Optional[dict]:
        """Create new config template.

        Args:
            path: path to config.

        Returns:
            Config.

        """
        if path is None:
            path = os.path.join(base_dir, cls._default_config_path)
            with open(path) as f:
                params = yaml.safe_load(f)
            return params

        else:
            shutil.copy(os.path.join(base_dir, cls._default_config_path), path)

    def create_automl(self, **fit_args):
        """Abstract method - how to build automl.

        Here you should create all automl components, like readers, levels, timers, blenders.
        Method ._initialize should be called in the end to create automl.

        Args:
            **fit_args: params that are passed to .fit_predict method.

        """
        raise NotImplementedError

    def fit_predict(self, train_data: Any, roles: dict, train_features: Optional[Sequence[str]] = None,
                    cv_iter: Optional[Iterable] = None,
                    valid_data: Optional[Any] = None,
                    valid_features: Optional[Sequence[str]] = None) -> LAMLDataset:
        """

        Args:
            train_data:  dataset to train.
            roles: roles dict.
            train_features: optional features names, if cannot be inferred from train_data.
            cv_iter: custom cv iterator. Ex. TimeSeriesIterator instance.
            valid_data: optional validation dataset.
            valid_features: optional validation dataset features if cannot be inferred from valid_data.

        Returns:
            LAMLDataset of predictions. Call .data to get predictions array.

        """
        self.create_automl(train_data=train_data,
                           roles=roles,
                           train_features=train_features,
                           cv_iter=cv_iter,
                           valid_data=valid_data,
                           valid_features=valid_features
                           )
        logger.info('Start automl preset with listed constraints:')
        logger.info('- time: {} seconds'.format(self.timer.timeout))
        logger.info('- cpus: {} cores'.format(self.cpu_limit))
        logger.info('- memory: {} gb'.format(self.memory_limit))
        logger.info('\n')
        self.timer.start()
        result = super().fit_predict(train_data, roles, train_features, cv_iter, valid_data, valid_features)
        self.timer.stop()

        logger.info('\n')
        logger.info('Automl preset training completed in {:.2f} seconds.'.format(self.timer.total_duration))

        return result

==> LightAutoML-0.2.8-py3-none-any.whl: lightautoml/automl/presets/base.py <==
"""AutoML presets base class."""

import os
import shutil
from typing import Optional, Any, Sequence, Iterable

import logging
import torch
import yaml
from log_calls import record_history

from ..base import AutoML
from ...dataset.base import LAMLDataset
from ...tasks import Task
from ...utils.logging import get_logger, verbosity_to_loglevel
from ...utils.timer import PipelineTimer

logger = get_logger(__name__)

base_dir = os.path.dirname(__file__)


@record_history(enabled=False)
def upd_params(old: dict, new: dict) -> dict:
    for k in new:
        if type(new[k]) is dict and k in old and type(old[k]) is dict:
            upd_params(old[k], new[k])
        else:
            old[k] = new[k]

    return old


@record_history(enabled=False)
class AutoMLPreset(AutoML):
    """ Basic class for automl preset.

    It's almost like AutoML, but with delayed initialization.
    Initialization starts on fit, some params are inferred from data
    Preset should be defined via .create_automl method. Params should be set via yaml config
    Most usefull case - end-to-end model development, ex:

    Example:

        >>> automl = SomePreset(Task('binary'), timeout=3600)
        >>> automl.fit_predict(data, roles={'target': 'TARGET'})

    """
    _default_config_path = 'example_config.yml'

    def __init__(self, task: Task, timeout: int = 3600, memory_limit: int = 16, cpu_limit: int = 4,
                 gpu_ids: Optional[str] = 'all', verbose: int = 2,
                 timing_params: Optional[dict] = None,
                 config_path: Optional[str] = None, **kwargs: Any):
        """

        Commonly _params kwargs (ex. timing_params) set via config file (config_path argument).
        If you need to change just few params, it's possible to pass it as dict of dicts, like json
        To get available params please look on default config template. Also you can find there param description
        To generate config template call SomePreset.get_config(config_path.yml)

        Args:
            task: Task to solve.
            timeout: timeout in seconds.
            memory_limit: memory limit that are passed to each automl.
            cpu_limit: cpu limit that that are passed to each automl.
            gpu_ids: gpu_ids that are passed to each automl.
            verbose: verbosity level that are passed to each automl.
            timing_params: timing param dict. Optional.
            config_path: path to config file.
            **kwargs:

        """
        self._set_config(config_path)
        logging.getLogger().setLevel(verbosity_to_loglevel(verbose))

        for name, param in zip(['timing_params'], [timing_params]):
            if param is None:
                param = {}
            self.__dict__[name] = {**self.__dict__[name], **param}

        self.timer = PipelineTimer(timeout, **self.timing_params)
        self.memory_limit = memory_limit
        if cpu_limit == -1:
            cpu_limit = os.cpu_count()
        self.cpu_limit = cpu_limit
        self.gpu_ids = gpu_ids
        if gpu_ids == 'all':
            self.gpu_ids = ','.join(map(str, range(torch.cuda.device_count())))
        self.task = task

        self.verbose = verbose

    def _set_config(self, path):

        if path is None:
            path = os.path.join(base_dir, self._default_config_path)

        with open(path) as f:
            params = yaml.safe_load(f)

        for k in params:
            self.__dict__[k] = params[k]

    @classmethod
    def get_config(cls, path: Optional[str] = None) -> Optional[dict]:
        """Create new config template.

        Args:
            path: path to config.

        Returns:
            Config.

        """
        if path is None:
            path = os.path.join(base_dir, cls._default_config_path)
            with open(path) as f:
                params = yaml.safe_load(f)
            return params

        else:
            shutil.copy(os.path.join(base_dir, cls._default_config_path), path)

    def create_automl(self, **fit_args):
        """Abstract method - how to build automl.

        Here you should create all automl components, like readers, levels, timers, blenders.
        Method ._initialize should be called in the end to create automl.

        Args:
            **fit_args: params that are passed to .fit_predict method.

        """
        raise NotImplementedError

    def fit_predict(self, train_data: Any, roles: dict, train_features: Optional[Sequence[str]] = None,
                    cv_iter: Optional[Iterable] = None,
                    valid_data: Optional[Any] = None,
                    valid_features: Optional[Sequence[str]] = None) -> LAMLDataset:
        """

        Args:
            train_data:  dataset to train.
            roles: roles dict.
            train_features: optional features names, if cannot be inferred from train_data.
            cv_iter: custom cv iterator. Ex. TimeSeriesIterator instance.
            valid_data: optional validation dataset.
            valid_features: optional validation dataset features if cannot be inferred from valid_data.

        Returns:
            LAMLDataset of predictions. Call .data to get predictions array.

        """
        self.create_automl(train_data=train_data,
                           roles=roles,
                           train_features=train_features,
                           cv_iter=cv_iter,
                           valid_data=valid_data,
                           valid_features=valid_features
                           )
        logger.info('Start automl preset with listed constraints:')
        logger.info('- time: {} seconds'.format(self.timer.timeout))
        logger.info('- cpus: {} cores'.format(self.cpu_limit))
        logger.info('- memory: {} gb\n'.format(self.memory_limit))
        self.timer.start()
        result = super().fit_predict(train_data, roles, train_features, cv_iter, valid_data, valid_features)
        logger.info('\nAutoml preset training completed in {:.2f} seconds.'.format(self.timer.time_spent))

        return result
